# -*- coding: utf-8 -*-
# source: https://stackoverflow.com/a/31505798/6942666
import re
alphabets = "([A-Za-z])"
prefixes = "(Mr|mr|St|st|Mrs|mrs|Ms|ms|Dr|dr)[.]"
suffixes = "(Inc|Ltd|Jr|Sr|Co)"
starters = "(Mr|mr|Mrs|mrs|Ms|ms|Dr|dr|He\s|She\s|It\s|They\s|Their\s|Our\s|We\s|But\s|However\s|That\s|This\s|Wherever)"
acronyms = "([A-Z][.][A-Z][.](?:[A-Z][.])?)"
websites = "[.](com|net|org|io|gov)"
digits = "([0-9])"


def split_into_sentences(text):
    text = " " + text + "  "
    text = text.replace("\n", " ")
    text = re.sub(prefixes, "\\1<prd>", text)
    text = re.sub(websites, "<prd>\\1", text)
    if "Ph.D" in text:
        text = text.replace("Ph.D.", "Ph<prd>D<prd>")
    text = re.sub("\s" + alphabets + "[.] ", " \\1<prd> ", text)
    text = re.sub(acronyms + " " + starters, "\\1<stop> \\2", text)
    text = re.sub(alphabets + "[.]" + alphabets + "[.]" + alphabets + "[.]", "\\1<prd>\\2<prd>\\3<prd>", text)
    text = re.sub(alphabets + "[.]" + alphabets + "[.]", "\\1<prd>\\2<prd>", text)
    text = re.sub(" " + suffixes + "[.] " + starters, " \\1<stop> \\2", text)
    text = re.sub(" " + suffixes + "[.]", " \\1<prd>", text)
    text = re.sub(" " + alphabets + "[.]", " \\1<prd>", text)
    if "”" in text:
        text = text.replace(".”", "”.")
    if "\"" in text:
        text = text.replace(".\"", "\".")
    if "!" in text:
        text = text.replace("!\"", "\"!")
    if "?" in text:
        text = text.replace("?\"", "\"?")
    text = text.replace(".", ".<stop>")
    text = text.replace("?", "?<stop>")
    text = text.replace("!", "!<stop>")
    text = text.replace("<prd>", ".")
    text = re.sub(digits + "[.]" + digits, "\\1<prd>\\2", text)
    sentences = text.split("<stop>")
    # sentences = sentences[:-1]

    updated_sentences = []
    for sentence in sentences:
        cleaned = sentence.strip()
        if len(cleaned) != 0:
            updated_sentences.append(cleaned)

    return updated_sentences



import os
import argparse
import json
from pycontractions import Contractions  # expand contractions
from aiotext.lemmatize import lemmatize
import re  # regex
import string  # string helper Library
from aiotext.splitToSentences import split_into_sentences
import regex


class Cleaner:
    def __init__(self, options={'expand_contractions': True, 'strip_text_in_brackets': False, "combine_concatenations": False, }):
        self.options = options

        if options['expand_contractions']:
            print("Loading contractions dataset (this will take a while the first time)")

            # Load your favorite word2vec model
            self.cont = Contractions(api_key="glove-twitter-100")
            print("Contractions dataset downloaded")

            print("Training contractions model (this will take a while)")
            # prevents loading on first expand_texts call
            self.cont.load_models()
            print("Contraction model successfully trained")

    def expand_contractions(self, text):
        text = text.replace("’", "'")  # need to put in the correct apostrophe
        expanded_text = list(self.cont.expand_texts([text], precise=True))
        return expanded_text[0]

    def strip_brackets(self, text):
        # Remove strings in brackets
        # Eg. "This is a sentence (extra info) description."
        # Becomes "This is a sentence description."
        """ Remove brackets from text
            Matches (), [], {}

            Converts:
            'hello (there) you (my[best] friend) lets {dine } }' -> 'hello  you  lets  }'
        """

        brace_open_type = ""
        brace_pair = {
            '(': ')',
            '[': ']',
            '{': '}'
        }
        open_brace_list = list(brace_pair.keys())

        res = ""
        for c in text:
            if len(brace_open_type) == 0:
                # not opened
                if c in open_brace_list:
                    brace_open_type = c
                else:
                    res += c
            else:
                # opened
                if brace_pair[brace_open_type] == c:
                    brace_open_type = ""

        return res

    def combine_concatenations(self, text):
        # convert concatenated words into seperate words
        # georgetown-louisville becomes georgetown louisville
        if self.options['combine_concatenations']:
            # matches all types of dashes
            # https://www.compart.com/en/unicode/category/Pd
            text = regex.sub(r'\p{Pd}+', '', text)
        else:
            # matches all types of dashes
            # https://www.compart.com/en/unicode/category/Pd
            text = regex.sub(r'\p{Pd}+', ' ', text)

        return text

    def split_sentences(self, text):
        """
        Returns: list of sentences
        """
        # split into list of sentences
        return split_into_sentences(text)

    def remove_non_english(self, sentences):
        """
        Removes non-english words and all punctuation and numbers
        Removes extra white space
        """
        # remove all punctuation (removes non-english words too)
        # stripped = re.sub('[^a-zA-Z\s]*', '', stripped)

        # removes extra white spaces
        # stripped = re.sub('[ ]{2,}',' ', stripped)

        stripped_sentences = []
        for sentence in sentences:
            cleaned = re.sub('[ ]{2,}', ' ', re.sub('[^a-zA-Z\s]*', '', sentence))
            if len(cleaned) != 0:
                stripped_sentences.append(cleaned)

        return stripped_sentences

    def tokenize_sentences(self, sentences):
        # tokenize:
        # this will be a 2 dimensional list
        # [['this', 'is', 'sentence'],
        # ['this', 'is', 'another']
        # ['this', 'is', 'another']]
        tokenized_sentences = []
        for sentence in sentences:
            tokenized_sentences.append(sentence.split())
        return tokenized_sentences

    def lemmatize_sentences(self, tokenized_sentences):
        """
        Recieves
            Args: tokenized_sentences is of form
                [['this', 'is', 'sentence'],
                ['this', 'is', 'another']
                ['this', 'is', 'another']]

            Returns: lemmatized 2d list of same form
                [['this', 'is', 'sentenc'],
                ['this', 'is', 'anoth']
                ['this', 'is', 'anoth']]
        """
        lemmatized_sentences = []
        for sentence in tokenized_sentences:
            lemmatized_sentences.append(lemmatize(sentence))
        # lemmatized_sentences = [lemmatize(sentence) for sentence in tokenized_sentences]
        return lemmatized_sentences

    def clean(self, text):
        text = text.lower()

        if self.options["expand_contractions"]:
            # Expands it's -> it is
            text = self.expand_contractions(text)

        if self.options["strip_text_in_brackets"]:
            text = self.strip_brackets(text)

        text = self.combine_concatenations(text)
        sentences = self.split_sentences(text)
        cleaned_sentences = self.remove_non_english(sentences)
        tokenized_cleaned_sentences = self.tokenize_sentences(cleaned_sentences)
        lemmatized_tokenized_sentences = self.lemmatize_sentences(tokenized_cleaned_sentences)

        return lemmatized_tokenized_sentences



# Handles lemmatization
# Recieves a list of tokens and returns a lemmatized list of tokens
# This is needed because lemmatizer doesn't automatically recognize
# parts of speech on it's own and often fails when something is not
# a verb. For instance, it would make "us" into "u" and "does" into "doe"

import nltk
from nltk.corpus import wordnet as wn
from nltk.stem import WordNetLemmatizer


def lemmatize(tokens):
    # map between Penn Bank Tags and Word Net Tags
    tag_map = {
        'CC': None,  # coordin. conjunction (and, but, or)
        'CD': wn.NOUN,  # cardinal number (one, two)
        'DT': None,  # determiner (a, the)
        'EX': wn.ADV,  # existential ‘there’ (there)
        'FW': None,  # foreign word (mea culpa)
        'IN': wn.ADV,  # preposition/sub-conj (of, in, by)
        'JJ': wn.ADJ,  # adjective (yellow)
        'JJR': wn.ADJ,  # adj., comparative (bigger)
        'JJS': wn.ADJ,  # adj., superlative (wildest)
        'LS': None,  # list item marker (1, 2, One)
        'MD': None,  # modal (can, should)
        'NN': wn.NOUN,  # noun, sing. or mass (llama)
        'NNS': wn.NOUN,  # noun, plural (llamas)
        'NNP': wn.NOUN,  # proper noun, sing. (IBM)
        'NNPS': wn.NOUN,  # proper noun, plural (Carolinas)
        'PDT': wn.ADJ,  # predeterminer (all, both)
        'POS': None,  # possessive ending (’s )
        'PRP': None,  # personal pronoun (I, you, he)
        'PRP$': None,  # possessive pronoun (your, one’s)
        'RB': wn.ADV,  # adverb (quickly, never)
        'RBR': wn.ADV,  # adverb, comparative (faster)
        'RBS': wn.ADV,  # adverb, superlative (fastest)
        'RP': wn.ADJ,  # particle (up, off)
        'SYM': None,  # symbol (+,%, &)
        'TO': None,  # “to” (to)
        'UH': None,  # interjection (ah, oops)
        'VB': wn.VERB,  # verb base form (eat)
        'VBD': wn.VERB,  # verb past tense (ate)
        'VBG': wn.VERB,  # verb gerund (eating)
        'VBN': wn.VERB,  # verb past participle (eaten)
        'VBP': wn.VERB,  # verb non-3sg pres (eat)
        'VBZ': wn.VERB,  # verb 3sg pres (eats)
        'WDT': None,  # wh-determiner (which, that)
        'WP': None,  # wh-pronoun (what, who)
        'WP$': None,  # possessive (wh- whose)
        'WRB': None,  # wh-adverb (how, where)
        '$': None,  # dollar sign ($)
        '#': None,  # pound sign (#)
        '“': None,  # left quote (‘ or “)
        '”': None,  # right quote (’ or ”)
        '(': None,  # left parenthesis ([, (, {, <)
        ')': None,  # right parenthesis (], ), }, >)
        ',': None,  # comma (,)
        '.': None,  # sentence-final punc (. ! ?)
        ':': None,  # mid-sentence punc (: ; ... – -)
    }

    lemmatizer = WordNetLemmatizer()

    # returns list of tuples (word, tag) - penn bank style
    tag_tuples = nltk.pos_tag(tokens)

    # converts penn bank to word net
    penn_keys = list(tag_map.keys())
    wn_tags = []

    for (word, tag) in tag_tuples:
        if tag in penn_keys:
            wn_tags.append((word, tag_map[tag]))
        else:
            wn_tags.append((word, None))

    # cycles through all tagged words
    # if there is no wn tag, it does nothing
    # if there is a tag, it lemmatizes
    output = []
    for (word, tag) in wn_tags:
        if tag == None:
            output.append(word)
        else:
            output.append(lemmatizer.lemmatize(word, tag))

    return output



from aiotext.aiotext import Cleaner


def test_aiotext():
    """
    Test Cleaner. Run with pytest
    """

    text = "Call me Ishmael. Some years ago—never mind how long precisely—having "
    text += "little or no money in my purse, and nothing particular to interest me "
    text += "on shore, I thought I would sail about a little and see the watery part "
    text += "of the world. It is a way I have of driving off the spleen and "
    text += "regulating the circulation. Whenever I find myself growing grim about "
    text += "the mouth; whenever it is a damp, drizzly November in my soul; whenever "
    text += "I find myself involuntarily pausing before coffin warehouses, and "
    text += "bringing up the rear of every funeral I meet; and especially whenever "
    text += "my hypos get such an upper hand of me, that it requires a strong moral "
    text += "principle to prevent me from deliberately stepping into the street, and "
    text += "methodically knocking people’s hats off—then, I account it high time to "
    text += "get to sea as soon as I can. This is my substitute for pistol and ball. "
    text += "With a philosophical flourish Cato throws himself upon his sword; I "
    text += "quietly take to the ship. There is nothing surprising in this. If they "
    text += "but knew it, almost all men in their degree, some time or other, "
    text += "cherish very nearly the same feelings towards the ocean with me."

    # Initialize cleaner
    cleaner_options = {
        "expand_contractions": False,
        "strip_text_in_brackets": False,
        "combine_concatenations": False,  # if false - seperates with space
    }
    cleaner = Cleaner(cleaner_options)

    assert cleaner.clean(text) == [
        ['call', 'me', 'ishmael'],
        ['some', 'year', 'ago', 'never', 'mind', 'how', 'long', 'precisely', 'have', 'little', 'or', 'no', 'money', 'in', 'my', 'purse', 'and', 'nothing', 'particular',
            'to', 'interest', 'me', 'on', 'shore', 'i', 'think', 'i', 'would', 'sail', 'about', 'a', 'little', 'and', 'see', 'the', 'watery', 'part', 'of', 'the', 'world'],
        ['it', 'be', 'a', 'way', 'i', 'have', 'of', 'drive', 'off',
            'the', 'spleen', 'and', 'regulate', 'the', 'circulation'],
        ['whenever', 'i', 'find', 'myself', 'grow', 'grim', 'about', 'the', 'mouth', 'whenever', 'it', 'be', 'a', 'damp', 'drizzly', 'november', 'in', 'my', 'soul', 'whenever', 'i', 'find', 'myself', 'involuntarily', 'pause', 'before', 'coffin', 'warehouse', 'and', 'bring', 'up', 'the', 'rear', 'of', 'every', 'funeral', 'i', 'meet', 'and', 'especially', 'whenever', 'my',
            'hypo', 'get', 'such', 'an', 'upper', 'hand', 'of', 'me', 'that', 'it', 'require', 'a', 'strong', 'moral', 'principle', 'to', 'prevent', 'me', 'from', 'deliberately', 'step', 'into', 'the', 'street', 'and', 'methodically', 'knock', 'people', 'hat', 'off', 'then', 'i', 'account', 'it', 'high', 'time', 'to', 'get', 'to', 'sea', 'as', 'soon', 'as', 'i', 'can'],
        ['this', 'be', 'my', 'substitute', 'for', 'pistol', 'and', 'ball'],
        ['with', 'a', 'philosophical', 'flourish', 'cato', 'throw', 'himself',
            'upon', 'his', 'sword', 'i', 'quietly', 'take', 'to', 'the', 'ship'],
        ['there', 'be', 'nothing', 'surprising', 'in', 'this'],
        ['if', 'they', 'but', 'know', 'it', 'almost', 'all', 'men', 'in', 'their', 'degree', 'some', 'time', 'or',
            'other', 'cherish', 'very', 'nearly', 'the', 'same', 'feeling', 'towards', 'the', 'ocean', 'with', 'me']
    ]

    text = "This is A sentence With wierd caps and no period"

    assert cleaner.clean(text) == [['this', 'be', 'a', 'sentence', 'with', 'wierd', 'cap', 'and', 'no', 'period']]

    # Initialize cleaner
    cleaner_options = {
        "expand_contractions": True,
        "strip_text_in_brackets": True,
        "combine_concatenations": True,  # if false - seperates with space
    }
    cleaner = Cleaner(cleaner_options)

    text = "This is a dog's bone. That's a cat. Here are some random people's possesives. This sentence doesn't make much sense. That's okay. Just want to give ya'll an example"

    print(cleaner.clean(text))
    assert cleaner.clean(text) == [['this', 'be', 'a', 'dog', 'bone'], ['that', 'be', 'a', 'cat'], ['here', 'be', 'some', 'random', 'people', 'possesives'], ['this', 'sentence', 'do', 'not', 'make', 'much', 'sense'], ['that', 'have', 'okay'], ['just', 'want', 'to', 'give', 'yall', 'an', 'example']]

    text = "Now let's do it all (here's something). Oh look more-brackets {yep here's more}. You guessed it. This man's math is right-here [5 + 5 * (2 * 2)]. Doesn't this make everything easier."

    print(cleaner.clean(text))
    assert cleaner.clean(text) == [['now', 'let', 'us', 'do', 'it', 'all'], ['oh', 'look', 'morebrackets'], ['you', 'guess', 'it'], ['this', 'man', 'math', 'be', 'righthere'], ['do', 'not', 'this', 'make', 'everything', 'easy']]


test_aiotext()
